#!/usr/bin/python
# coding=utf-8
import logging
import logging.handlers
import sys

DEFAULT_LOG_MAX_BYTES = 1024*1024

CRITICAL = "critical"
DEBUG = "debug"
ERROR = "error"
EXCEPTION = "exception"
INFO = "info"
WARN = "warn"

ERROR_WRITE_LOG_EXIT_CODE = 105

class BaseLogging:
    # this class has global handler members of stream, file and http types
    __base_logger = logging.getLogger("main")
    __formatter = logging.Formatter("%(asctime)s - %(name)s: %(lineno)s - %(levelname)s - %(message)s")
    __file_handler = None
    __http_handler = logging.handlers.HTTPHandler("", "", "POST")
    __stream_hander = logging.StreamHandler()
    __logFile = None

    # by default, the global logger will not add any handler
    def __init__(self):
        self.__stream_hander.setFormatter(self.__formatter)
        self.__base_logger.setLevel(logging.INFO)

    def getLogger(self):
        return self.__base_logger

    def listHandlers(self):
        return self.__base_logger.handlers

    # add the file handler to the global logger
    def addFileHandler(self, logFile=None):
        try:
            self.__logFile=logFile
            if logFile != None:
                self.__file_handler = logging.handlers.RotatingFileHandler(logFile,
                                                                      maxBytes=DEFAULT_LOG_MAX_BYTES,
                                                                      backupCount=3)
            else:
                self.__file_handler = logging.handlers.RotatingFileHandler("/tmp/daemon.log",
                                                                       maxBytes=DEFAULT_LOG_MAX_BYTES,
                                                                       backupCount=10)
            self.__file_handler.setFormatter(self.__formatter)
            self.__base_logger.addHandler(self.__file_handler)
        except Exception as e:
            sys.stderr.write("init log file {} error: {}".format(logFile, str(e)))
            sys.exit(ERROR_WRITE_LOG_EXIT_CODE)

    # remove the file handler from the global logger
    def removeFileHandler(self):
        self.__base_logger.removeHandler(self.__file_handler)

    # add the http handler to the global logger
    def addHttpHandler(self):
        self.__base_logger.addHandler(self.__http_handler)

    # set the host and url of the global http handler
    def setHttpHandler(self, host, url):
        self.__http_handler.host = host
        self.__http_handler.url = url

    # reset the host and url of the global http handler
    def resetHttpHandler(self):
        self.__http_handler.host = ""
        self.__http_handler.url = ""

    # remove the http handler from the global logger
    def removeHttpHandler(self):
        self.__base_logger.removeHandler(self.__http_handler)

    # add the stream handler to the global logger
    def addStreamHandler(self):
        self.__base_logger.addHandler(self.__stream_hander)

    # remove the stream handler from the global logger
    def removeStreamHandler(self):
        self.__base_logger.removeHandler(self.__stream_hander)

    def doLogging(self, content, log_type=DEBUG):
        try:
            if log_type == INFO:
                self.__base_logger.info(content)
            elif log_type == ERROR:
                self.__base_logger.error(content)
            elif log_type == CRITICAL:
                self.__base_logger.critical(content)
            elif log_type == EXCEPTION:
                self.__base_logger.exception(content)
            else:
                self.__base_logger.debug(content)
            return True
        except Exception as e:
            sys.stderr.write("init log file {} error: {}".format(self.__logFile, str(e)))
            sys.exit(ERROR_WRITE_LOG_EXIT_CODE)
