# -*- coding: utf-8 -*-
import sys
if sys.getdefaultencoding() != 'utf-8':     # 重新设置python 编码为utf-8
    reload(sys)
    sys.setdefaultencoding('utf-8')
from baseLogging import *
import mysql.connector
import json
import sys
from datetime import datetime

__logger = BaseLogging()

EXECUTION_STOP_INFO_PATH = '/tmp/execution.{}'

def CreateTable(config, mysql_table):
    sql_str = "CREATE TABLE if not exists `{}` ( \
                  `item` varchar(255) NOT NULL DEFAULT '', \
                  `time` datetime NOT NULL, \
                  `title` varchar(255) NOT NULL DEFAULT '', \
                  `urls` varchar(255) NOT NULL, \
                  `clicks` int(11) DEFAULT NULL, \
                  `last_clicks` int(11) DEFAULT NULL, \
                  `new_clicks` int(11) DEFAULT NULL, \
                  PRIMARY KEY (`urls`,`time`) \
               ) ENGINE=InnoDB AUTO_INCREMENT=479460 DEFAULT CHARSET=utf8mb4;".format(mysql_table)
    mysql_exec(config, sql_str)

def DeleteRecord(config, time, expr, type, table):
    sql_str = "delete from {} where date_add(time,interval {} {})<='{}'".format(table, expr, type, time)
    mysql_exec(config, sql_str)

def exec_fail(message, resultFile):
    __logger.doLogging(message, ERROR)
    result = {}
    result["message"] = message
    result["result"] = -1
    writeResult(resultFile, result)
    sys.exit(100)

def writeResult(resultFile, resultDict):
    content = json.dumps(resultDict, sort_keys=True, indent=4, separators=(',', ': '))
    try:
        with open(resultFile, 'w') as f:
            f.write(content)
    except Exception as err:
        sys.exit(100)

def readFileToJson(file_name):
    with open(file_name) as rf:
        contents = rf.read()
    return contents

def checkTask(task_parameter):
    if not task_parameter or len(task_parameter) == 0:
        sys.stderr.write("parameter file is empty,please check")
        sys.exit(100)
    taskDict = json.loads(readFileToJson(task_parameter))
    return taskDict

def getReqParmsAfterGetLogPath(field, parentDict, resultFile):
    if field in parentDict and len(str(parentDict[field])) > 0:
        rt = parentDict[field]
        __logger.doLogging("{} : {}".format(field, rt), INFO)
        return rt
    else:
        message = "{} cat not be empty".format(field)
        exec_fail(message, resultFile)

def getUnreqParmsAfterGetLogPath(field, parentDict, default):
    rt = default
    if field in parentDict and parentDict[field] is not None and len(str(parentDict[field])) > 0:
        rt = parentDict[field]
        __logger.doLogging("{} : {}".format(field, rt), INFO)
    else:
        message = "{} is empty, ignore it".format(field)
        __logger.doLogging(message, INFO)
    return rt

def mysql_exec(config, sql_str):
    try:
        conn = mysql.connector.connect(**config)
    except mysql.connector.Error as e:
        __logger.doLogging('connect fails! {}'.format(e), ERROR)
        raise
    cursor = conn.cursor()
    try:
        __logger.doLogging('execute sql: {}'.format(sql_str), INFO)
        cursor.execute(sql_str)
        conn.commit()
    except mysql.connector.Error as e:
        __logger.doLogging('execute fails! xxxx{}'.format(e), ERROR)
        raise
    finally:
        cursor.close()
        conn.close()

def run(input):
    dict = checkTask(input)
    logFile = dict["logPath"]
    __logger.addFileHandler(logFile)
    executionId = dict.get("executionId")
    __logger.doLogging("start to execute ...", INFO)
    parameters = dict["parameter"]
    resultFile = dict["resultPath"]
    dt = datetime.now()

    mysql_host = getUnreqParmsAfterGetLogPath(field="MYSQL_HOST", parentDict=parameters, default="linktime-mysql.marathon.mesos")
    mysql_port = getUnreqParmsAfterGetLogPath(field="MYSQL_PORT", parentDict=parameters, default="3306")
    mysql_uid = getUnreqParmsAfterGetLogPath(field="MYSQL_UID", parentDict=parameters, default="root")
    mysql_passwd = getUnreqParmsAfterGetLogPath(field="MYSQL_PASSWD", parentDict=parameters, default="123456")
    mysql_database = getUnreqParmsAfterGetLogPath(field="MYSQL_DATABASE", parentDict=parameters, default="demo")
    mysql_table = getUnreqParmsAfterGetLogPath(field="MYSQL_TABLE", parentDict=parameters, default="demouser01_newsAdd2")
    expr = getUnreqParmsAfterGetLogPath(field="EXPR", parentDict=parameters, default='3')
    type = getUnreqParmsAfterGetLogPath(field="TYPE", parentDict=parameters, default='day')

    time = getUnreqParmsAfterGetLogPath(field="TIME_PARTITION", parentDict=parameters, default=dt.strftime('%Y-%m-%d %H:00:00'))
    config = {
        'host': mysql_host,
        'user': mysql_uid,
        'password': mysql_passwd,
        'port': mysql_port,
        'database': mysql_database,
        'charset': 'utf8'
    }

    __logger.doLogging('mysql info: {}'.format(config), INFO)
    result = {}

    try:
        CreateTable(config, mysql_table)
        DeleteRecord(config, time, expr, type, mysql_table)
        result["result"] = 1
        result["message"] = "success"
        writeResult(resultFile, result)
    except Exception as e:
        message = "exec failed {}".format(e)
        exec_fail(message, resultFile)

    return input

if __name__ == '__main__':
    try:
        result = run(sys.argv[1])
    except Exception, re:
        pass